package com.flyfish.framework.dict.config;

import com.flyfish.framework.annotations.DictValue;
import com.flyfish.framework.dict.annotations.EnableDictProcess;
import com.flyfish.framework.dict.domain.Dictionary;
import com.flyfish.framework.dict.domain.DictionaryValue;
import com.flyfish.framework.dict.service.DictionaryService;
import com.flyfish.framework.enums.BlankEnum;
import com.flyfish.framework.enums.NamedEnum;
import com.flyfish.framework.utils.Assert;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.reflections.Reflections;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.context.annotation.ImportBeanDefinitionRegistrar;
import org.springframework.core.type.AnnotationMetadata;

import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 字典处理器
 *
 * @author wangyu
 * 用于反向注册默认值
 */
@RequiredArgsConstructor
public class DictionaryProcessor implements ImportBeanDefinitionRegistrar {

    private final DictionaryService dictionaryService;

    private String[] basePackages;

    /**
     * spring加载完成后，尝试注入值
     */
    public void initialize() {
        Assert.notNull(basePackages, "未指定明确的字典扫描路径！");
        Reflections reflections = new Reflections(Stream.of(basePackages).collect(Collectors.toSet()));
        Set<Field> fields = reflections.getFieldsAnnotatedWith(DictValue.class);
        if (CollectionUtils.isNotEmpty(fields)) {
            // 查找是否存在，不存在插入，存在无视
            List<Dictionary> dictionaries = fields.stream().map(field -> field.getAnnotation(DictValue.class))
                    .filter(annotation -> null != annotation && StringUtils.isNotBlank(annotation.value()) &&
                            BlankEnum.class != annotation.enumType())
                    .map(annotation -> dictionaryService
                            .getByCode(annotation.value())
                            .map(dictionary -> {
                                if (CollectionUtils.isEmpty(dictionary.getValues())) {
                                    dictionary.setValues(mapValues(annotation.enumType()));
                                }
                                return dictionary;
                            })
                            .orElseGet(() -> {
                                Dictionary dictionary = new Dictionary();
                                dictionary.setCode(annotation.value());
                                dictionary.setValues(mapValues(annotation.enumType()));
                                dictionary.setName(StringUtils.isNotBlank(annotation.name()) ? annotation.name() : annotation.value());
                                return dictionary;
                            }))
                    .distinct()
                    .collect(Collectors.toList());
            dictionaryService.updateBatch(dictionaries);
        }
    }

    /**
     * 映射值，转换为字典表的值
     *
     * @return 结果
     */
    private List<DictionaryValue> mapValues(Class<? extends NamedEnum> enumClass) {
        return Arrays.stream(enumClass.getEnumConstants()).map(item -> {
            DictionaryValue value = new DictionaryValue();
            value.setEnable(true);
            value.setText(item.getName());
            value.setValue(((Enum<?>) item).name());
            return value;
        }).collect(Collectors.toList());
    }

    /**
     * 注册bean定义，可以从这里取得元数据，自动加载bean
     *
     * @param metadata 元数据
     * @param registry 注册器
     */
    @Override
    public void registerBeanDefinitions(AnnotationMetadata metadata, BeanDefinitionRegistry registry) {
        Map<String, Object> attrs = metadata.getAnnotationAttributes(EnableDictProcess.class.getCanonicalName(), true);
        this.basePackages = (String[]) MapUtils.getObject(attrs, "basePackages");
        this.initialize();
    }
}
